// Copyright 2014 Citra Emulator Project
// Licensed under GPLv2 or any later version
// Refer to the license.txt file included.

#include "common/archives.h"
#include "core/hle/service/apt/apt_a.h"

namespace Service::APT {

APT_A::APT_A(std::shared_ptr<Module> apt)
    : Module::APTInterface(std::move(apt), "APT:A", MaxAPTSessions) {
    static const FunctionInfo functions[] = {
        // clang-format off
        {IPC::MakeHeader(0x0001, 1, 0), &APT_A::GetLockHandle, "GetLockHandle"},
        {IPC::MakeHeader(0x0002, 2, 0), &APT_A::Initialize, "Initialize"},
        {IPC::MakeHeader(0x0003, 1, 0), &APT_A::Enable, "Enable"},
        {IPC::MakeHeader(0x0004, 1, 0), nullptr, "Finalize"},
        {IPC::MakeHeader(0x0005, 1, 0), &APT_A::GetAppletManInfo, "GetAppletManInfo"},
        {IPC::MakeHeader(0x0006, 1, 0), &APT_A::GetAppletInfo, "GetAppletInfo"},
        {IPC::MakeHeader(0x0007, 0, 0), nullptr, "GetLastSignaledAppletId"},
        {IPC::MakeHeader(0x0008, 0, 0), nullptr, "CountRegisteredApplet"},
        {IPC::MakeHeader(0x0009, 1, 0), &APT_A::IsRegistered, "IsRegistered"},
        {IPC::MakeHeader(0x000A, 1, 0), nullptr, "GetAttribute"},
        {IPC::MakeHeader(0x000B, 1, 0), &APT_A::InquireNotification, "InquireNotification"},
        {IPC::MakeHeader(0x000C, 4, 4), &APT_A::SendParameter, "SendParameter"},
        {IPC::MakeHeader(0x000D, 2, 0), &APT_A::ReceiveParameter, "ReceiveParameter"},
        {IPC::MakeHeader(0x000E, 2, 0), &APT_A::GlanceParameter, "GlanceParameter"},
        {IPC::MakeHeader(0x000F, 4, 0), &APT_A::CancelParameter, "CancelParameter"},
        {IPC::MakeHeader(0x0010, 3, 2), nullptr, "DebugFunc"},
        {IPC::MakeHeader(0x0011, 3, 0), nullptr, "MapProgramIdForDebug"},
        {IPC::MakeHeader(0x0012, 1, 0), nullptr, "SetHomeMenuAppletIdForDebug"},
        {IPC::MakeHeader(0x0013, 0, 0), nullptr, "GetPreparationState"},
        {IPC::MakeHeader(0x0014, 1, 0), nullptr, "SetPreparationState"},
        {IPC::MakeHeader(0x0015, 5, 0), &APT_A::PrepareToStartApplication, "PrepareToStartApplication"},
        {IPC::MakeHeader(0x0016, 1, 0), &APT_A::PreloadLibraryApplet, "PreloadLibraryApplet"},
        {IPC::MakeHeader(0x0017, 1, 0), &APT_A::FinishPreloadingLibraryApplet, "FinishPreloadingLibraryApplet"},
        {IPC::MakeHeader(0x0018, 1, 0), &APT_A::PrepareToStartLibraryApplet, "PrepareToStartLibraryApplet"},
        {IPC::MakeHeader(0x0019, 1, 0), &APT_A::PrepareToStartSystemApplet, "PrepareToStartSystemApplet"},
        {IPC::MakeHeader(0x001A, 0, 0), &APT_A::PrepareToStartNewestHomeMenu, "PrepareToStartNewestHomeMenu"},
        {IPC::MakeHeader(0x001B, 3, 4), &APT_A::StartApplication, "StartApplication"},
        {IPC::MakeHeader(0x001C, 0, 0), &APT_A::WakeupApplication, "WakeupApplication"},
        {IPC::MakeHeader(0x001D, 0, 0), nullptr, "CancelApplication"},
        {IPC::MakeHeader(0x001E, 2, 4), &APT_A::StartLibraryApplet, "StartLibraryApplet"},
        {IPC::MakeHeader(0x001F, 2, 4), &APT_A::StartSystemApplet, "StartSystemApplet"},
        {IPC::MakeHeader(0x0020, 1, 4), nullptr, "StartNewestHomeMenu"},
        {IPC::MakeHeader(0x0021, 0, 0), &APT_A::OrderToCloseApplication, "OrderToCloseApplication"},
        {IPC::MakeHeader(0x0022, 1, 0), &APT_A::PrepareToCloseApplication, "PrepareToCloseApplication"},
        {IPC::MakeHeader(0x0023, 1, 0), nullptr, "PrepareToJumpToApplication"},
        {IPC::MakeHeader(0x0024, 1, 4), nullptr, "JumpToApplication"},
        {IPC::MakeHeader(0x0025, 3, 0), &APT_A::PrepareToCloseLibraryApplet, "PrepareToCloseLibraryApplet"},
        {IPC::MakeHeader(0x0026, 0, 0), &APT_A::PrepareToCloseSystemApplet, "PrepareToCloseSystemApplet"},
        {IPC::MakeHeader(0x0027, 1, 4), &APT_A::CloseApplication, "CloseApplication"},
        {IPC::MakeHeader(0x0028, 1, 4), &APT_A::CloseLibraryApplet, "CloseLibraryApplet"},
        {IPC::MakeHeader(0x0029, 1, 4), &APT_A::CloseSystemApplet, "CloseSystemApplet"},
        {IPC::MakeHeader(0x002A, 0, 0), &APT_A::OrderToCloseSystemApplet, "OrderToCloseSystemApplet"},
        {IPC::MakeHeader(0x002B, 0, 0), &APT_A::PrepareToJumpToHomeMenu, "PrepareToJumpToHomeMenu"},
        {IPC::MakeHeader(0x002C, 1, 4), &APT_A::JumpToHomeMenu, "JumpToHomeMenu"},
        {IPC::MakeHeader(0x002D, 0, 0), &APT_A::PrepareToLeaveHomeMenu, "PrepareToLeaveHomeMenu"},
        {IPC::MakeHeader(0x002E, 1, 4), &APT_A::LeaveHomeMenu, "LeaveHomeMenu"},
        {IPC::MakeHeader(0x002F, 1, 0), nullptr, "PrepareToLeaveResidentApplet"},
        {IPC::MakeHeader(0x0030, 1, 4), nullptr, "LeaveResidentApplet"},
        {IPC::MakeHeader(0x0031, 4, 0), &APT_A::PrepareToDoApplicationJump, "PrepareToDoApplicationJump"},
        {IPC::MakeHeader(0x0032, 2, 4), &APT_A::DoApplicationJump, "DoApplicationJump"},
        {IPC::MakeHeader(0x0033, 0, 0), &APT_A::GetProgramIdOnApplicationJump, "GetProgramIdOnApplicationJump"},
        {IPC::MakeHeader(0x0034, 2, 4), nullptr, "SendDeliverArg"},
        {IPC::MakeHeader(0x0035, 2, 0), &APT_A::ReceiveDeliverArg, "ReceiveDeliverArg"},
        {IPC::MakeHeader(0x0036, 1, 0), &APT_A::LoadSysMenuArg, "LoadSysMenuArg"},
        {IPC::MakeHeader(0x0037, 1, 2), &APT_A::StoreSysMenuArg, "StoreSysMenuArg"},
        {IPC::MakeHeader(0x0038, 1, 0), nullptr, "PreloadResidentApplet"},
        {IPC::MakeHeader(0x0039, 1, 0), nullptr, "PrepareToStartResidentApplet"},
        {IPC::MakeHeader(0x003A, 1, 4), nullptr, "StartResidentApplet"},
        {IPC::MakeHeader(0x003B, 1, 0), &APT_A::CancelLibraryApplet, "CancelLibraryApplet"},
        {IPC::MakeHeader(0x003C, 1, 2), nullptr, "SendDspSleep"},
        {IPC::MakeHeader(0x003D, 1, 2), nullptr, "SendDspWakeUp"},
        {IPC::MakeHeader(0x003E, 2, 0), nullptr, "ReplySleepQuery"},
        {IPC::MakeHeader(0x003F, 1, 0), nullptr, "ReplySleepNotificationComplete"},
        {IPC::MakeHeader(0x0040, 1, 2), &APT_A::SendCaptureBufferInfo, "SendCaptureBufferInfo"},
        {IPC::MakeHeader(0x0041, 1, 0), &APT_A::ReceiveCaptureBufferInfo, "ReceiveCaptureBufferInfo"},
        {IPC::MakeHeader(0x0042, 2, 0), nullptr, "SleepSystem"},
        {IPC::MakeHeader(0x0043, 1, 0), &APT_A::NotifyToWait, "NotifyToWait"},
        {IPC::MakeHeader(0x0044, 0, 0), &APT_A::GetSharedFont, "GetSharedFont"},
        {IPC::MakeHeader(0x0045, 1, 0), &APT_A::GetWirelessRebootInfo, "GetWirelessRebootInfo"},
        {IPC::MakeHeader(0x0046, 4, 4), &APT_A::Wrap, "Wrap"},
        {IPC::MakeHeader(0x0047, 4, 4), &APT_A::Unwrap, "Unwrap"},
        {IPC::MakeHeader(0x0048, 4, 0), nullptr, "GetProgramInfo"},
        {IPC::MakeHeader(0x0049, 6, 0), nullptr, "Reboot"},
        {IPC::MakeHeader(0x004A, 1, 0), &APT_A::GetCaptureInfo, "GetCaptureInfo"},
        {IPC::MakeHeader(0x004B, 3, 2), &APT_A::AppletUtility, "AppletUtility"},
        {IPC::MakeHeader(0x004C, 0, 0), nullptr, "SetFatalErrDispMode"},
        {IPC::MakeHeader(0x004D, 2, 0), nullptr, "GetAppletProgramInfo"},
        {IPC::MakeHeader(0x004E, 0, 0), nullptr, "HardwareResetAsync"},
        {IPC::MakeHeader(0x004F, 2, 0), &APT_A::SetAppCpuTimeLimit, "SetAppCpuTimeLimit"},
        {IPC::MakeHeader(0x0050, 1, 0), &APT_A::GetAppCpuTimeLimit, "GetAppCpuTimeLimit"},
        {IPC::MakeHeader(0x0051, 2, 0), &APT_A::GetStartupArgument, "GetStartupArgument"},
        {IPC::MakeHeader(0x0052, 4, 4), nullptr, "Wrap1"},
        {IPC::MakeHeader(0x0053, 4, 4), nullptr, "Unwrap1"},
        {IPC::MakeHeader(0x0055, 1, 0), &APT_A::SetScreenCapPostPermission, "SetScreenCapPostPermission"},
        {IPC::MakeHeader(0x0056, 0, 0), &APT_A::GetScreenCapPostPermission, "GetScreenCapPostPermission"},
        {IPC::MakeHeader(0x0057, 1, 4), nullptr, "WakeupApplication2"},
        {IPC::MakeHeader(0x0058, 0, 2), nullptr, "GetProgramID"},
        {IPC::MakeHeader(0x0101, 0, 0), &APT_A::CheckNew3DSApp, "CheckNew3DSApp"},
        {IPC::MakeHeader(0x0102, 0, 0), &APT_A::CheckNew3DS, "CheckNew3DS"},
        {IPC::MakeHeader(0x0103, 0, 0), &APT_A::Unknown0x0103, "Unknown0x0103"},
        {IPC::MakeHeader(0x0104, 0, 0), nullptr, "IsStandardMemoryLayout"},
        {IPC::MakeHeader(0x0105, 4, 0), &APT_A::IsTitleAllowed, "IsTitleAllowed"},
        // clang-format on
    };
    RegisterHandlers(functions);
}

} // namespace Service::APT

SERIALIZE_EXPORT_IMPL(Service::APT::APT_A)
